/// t2c.h
/// Global definitions for T2C code generator

/////////////////////////////////////////////////////////////////////////////
// Copyright 2009-2010 
// Institute for System Programming of the Russian Academy of Sciences (ISPRAS)
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
//    http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License. 
/////////////////////////////////////////////////////////////////////////////

#ifndef T2C_H_1402_INCLUDED
#define T2C_H_1402_INCLUDED
    
#ifdef __cplusplus
extern "C" {
#endif
    
///////////////////////////////////////////////////////////////////////////
struct CStringMap_; // forward decl

/// An aggregate for program settings.
struct SSettings
{
    /// Path to the directory containing data required by t2c: 
    /// default templates, etc.
    char* t2c_data_dir;

    /// Main template directory. The templates for the files to be generated are
    /// loaded from here.
    /// The name of the requested target subdirectory is appended to this path.
    char* template_dir;
    
    /// Root directory of the test suite for a given subsystem.
    char* suite_root_dir;
    
    /// Path to the .t2c-file to be processed.
    /// If NULL, the program generates files that are common for the test 
    /// suite, files for the tests from this .t2c-file otherwise.
    char* t2c_file_path;
    
    /// Name of the .t2c-file to be processed with ".t2c" extension stripped 
    /// if it was present. NULL when generating common files.
    char* t2c_file_name;
    
    /// Name of the target configuration to generate files for.
    /// It is also the name of the relevant subdirectory in the template directory.
    char* target;
    
    /// A string map containing the parameters of the test group read from 
    /// the configuration file(s).
    struct CStringMap_* test_group_params;
    
    /// Path to the additional template directory for the suite (see PNAME_TEMPLATE_DIR)
    /// with target subdir appended.
    /// It can be specified in a conf. file (base path, i.e. without target) but is 
    /// an optional feature.
    /// NULL means "not provided".
    /// If the path to the directory specified in the conf. file is not an absolute path,
    /// it is considered relative to the root directory of the suite.
    char* aux_template_dir;
    
    /// Path to the directory containing the templates for a partucular t2c-file 
    /// ("per-test templates") for a given target. 
    /// These templates may override those from 'aux_template_dir'
    /// (if present) and from the main template directory.
    /// The path is deduced from the path to the .t2c file.
    /// NULL means "not provided".
    char* per_test_template_dir;
    
    /// A string map containing (<template_name>, <path_to_template_directory>) pairs.
    struct CStringMap_* template_list;
    
    /// Path to the file containing the list of names (or paths) to the tests.
    /// If not NULL and the respective file can be opened for reading, the names read 
    /// from it will be the values of 'TEST_MODULE' multivalued attribute. 
    /// Otherwise, 'TEST_MODULE' will remain undefined. 
    /// This attribute is available for both per-test and common mode but is mostly 
    /// useful in the latter.
    char* test_name_file_path;
};

///////////////////////////////////////////////////////////////////////////

/// Clean up the contents of a settings object: free memory, etc.
/// This function must be called before exiting the program normally or with 
/// an exit() call.
void 
cleanup_settings(struct SSettings* psettings);

/// Initialize the code generator: set global variables, process the arguments 
/// specified in the command line, etc.
/// Return nonzero if successful, 0 otherwise (particularly if there are too 
/// few or too much arguments or an unknown option is specified).
/// If '--help', '--version' or '--print-data-dir' options are encountered,
/// this function displays the requested information and the program exits with
/// EXIT_SUCCESS status.
int
init(int argc, char* argv[], struct SSettings* psettings);

/// Load the configuration parameters for the suite.
/// The function also adds standard parameters with appropriate values to the 
/// collection of parameters: SUITE_ROOT_DIR, SUITE_NAME, T2C_FILE_NAME.
/// [NB] T2C_FILE_NAME is set in per-test mode only.
/// The function returns nonzero if successful, 0 otherwise.
int
load_configuration(struct SSettings* psettings);

/// Create a  map <template_name, full_path_to_template_dir> and return a pointer
/// to it. NULL is returned in case of error.
/// First, the function processes the main template directory for the current 
/// target, populating the map with names of its subdirs and full paths to them. 
/// Then it processes the auxiliary template directory of the suite if it is 
/// specified. The 
/// If the program is operating in per-test mode (i.e. if a t2c file was specified
/// in the command line), the function then processes the per-test template directory
/// (if it exist for the current target).
/// On each step, if an element with a particular key (name of the template) already 
/// exists in the map, its value is overridden. Otherwise, a new element is added 
/// to the map. The other elements remain unchanged.
/// 
/// This is the way template inheritance and overriding is implemented here.
struct CStringMap_*
load_template_list(struct SSettings* psettings);

/// Generate output files.
/// The function returns nonzero if successful, 0 otherwise.
int
generate_files(struct SSettings* psettings);

// Like fprintf(stderr, format, ...) with the following exceptions:
// 1. A common prefix is output at the beginning. It usually looks like this:
//    "<app_name>: "
// 2. A newline is output at the end.
// [NB] Similar to fprintf, the variable argument list can be empty.
int 
print_error(const char* format, ...);

#ifdef __cplusplus
}
#endif
  
#endif //T2C_H_1402_INCLUDED
